package com.accusoft.projects;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.io.IOException;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import javax.net.ssl.HttpsURLConnection;

import java.nio.channels.*;
import java.util.concurrent.TimeUnit;

public class PrizmDocConverter {

    private String ApiKey;

    //get the API key settings from the config.json file
    public static String getAPIKey() {
        String key = null;
        try {
            JSONParser parser = new JSONParser();
            Object obj = parser.parse(new FileReader("config.json"));
            JSONObject readConfig = (JSONObject) obj;
            // now get the API key
            key = (String) readConfig.get("apiKey");
        } catch (IOException e) {
            // do something with the exception
            e.printStackTrace();
            System.out.println(e);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return key;
    }

    public PrizmDocConverter(String newApiKey) {
        if (newApiKey != null || newApiKey == "") {
            this.ApiKey = newApiKey;
            // System.out.println("The API key is "+this.ApiKey);
        } else {
            System.out.println("APIKey is empty.");
        }
    }

    /*
	 * getApiKey
	 * Parameters:  None
	 * Returns the class's API key as a string
     */
    public String getApiKey() {
        return this.ApiKey;
    }

    public boolean IsApiBlank() {
        if (this.ApiKey == null || this.ApiKey == "") {
            return true;
        } else {
            return false;
        }
    }

    /*
     * WorkFileUpload
     * 
     * Uploads a file via the Prizmdoc Work File API.
     * The URL is hard coded for now.
     *
     * Parameters:
     * String uploadFile - the file name that is going to be uploaded
     *
     * See Accusoft Prizm Cloud Based documentation for more details:
     *
     * http://help.accusoft.com/SAAS/pcc-for-acs/work-files.html#get-data
     *
     * Return: a JSON string that includes the fileId and affinityToken.
     * 
     *
     *
     */
    public String WorkFileUpload(String uploadFrom) {
        
        DataOutputStream wr = null;
        FileInputStream inputStream = null;
        String response = null;
        try {
            URL workFileURL = new URL("https://api.accusoft.com/PCCIS/V1/WorkFile");
            HttpsURLConnection workFileConnection = (HttpsURLConnection) workFileURL.openConnection();
            workFileConnection.setRequestMethod("POST");
            workFileConnection.setRequestProperty("acs-api-key", getAPIKey());
            workFileConnection.setRequestProperty("Content-Type", "application/octet-stream");
            // Send post request
            workFileConnection.setDoOutput(true);
            wr = new DataOutputStream(workFileConnection.getOutputStream());
            // open our file
            // read it into our stream and upload it
            inputStream = new FileInputStream(uploadFrom);
            int bytesRead = -1;
            while ((bytesRead = inputStream.read()) != -1) {
                wr.write(bytesRead);
            }
            wr.flush();
            inputStream.close();
            wr.close();
            //Read the response
            BufferedReader in = new BufferedReader(
                    new InputStreamReader(workFileConnection.getInputStream()));
            String inputLine;
            StringBuffer responseBuff = new StringBuffer();

            while ((inputLine = in.readLine()) != null) {
                responseBuff.append(inputLine);
            }
            response = responseBuff.toString();
            in.close();
        } catch (ProtocolException e) {
            e.printStackTrace();
        } catch (MalformedURLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return response;
    }

    /*
     * ContentConvert:
     * Submits a Conversion request via the Accusoft PrizmDoc
     * Content Conversion API.
     *
     * Please see the Accusoft PrizmDoc Content Conversion API
     * documentation for more details:
     *
     * http://help.accusoft.com/SAAS/pcc-for-acs/webframe.html#content-conversion-service.html
     *
     *
     * Parameters:
     * String affinityToken:  the Accusoft-Affinity-Token returned as part of the
     *                  JSON object in the Work File API upload
     * 
     * String filedId:  the fileId returned as part of the JSON object in the
     *                  Work File API upload
     * String format:   The format of the file to convert to. 
     *
     * Returns:  a JSON object with the details of the file conversion process,
     *           including the processId
     */
    public String ContentConvert(String affinityToken, String fileId, String format) {
        // make sure the API key isn't blank
        if (IsApiBlank()) {

            return "API key is blank, please start again.";
        }

        // the URI to upload
        String url = "https://api.accusoft.com/v2/contentConverters";

        // set the headers
        try {
            URL workFileURL = new URL(url);

            HttpsURLConnection workFileConnection = (HttpsURLConnection) workFileURL.openConnection();
            workFileConnection.setRequestMethod("POST");
            workFileConnection.setRequestProperty("acs-api-key", this.ApiKey);
            workFileConnection.setRequestProperty("Content-Type", "application/json");
            workFileConnection.setRequestProperty("Accusoft-Affinity-Token", affinityToken);

            // create the JSON object
            JSONObject fileIds = new JSONObject();
            fileIds.put("fileId", fileId);

            // the formats object
            JSONObject formats = new JSONObject();
            formats.put("format", format);

            // if we had more then one file we could put them into the array of fileIds
            JSONArray fileIdArray = new JSONArray();
            fileIdArray.add(fileIds);

            // the source object
            JSONObject sources = new JSONObject();
            sources.put("sources", fileIdArray);
            sources.put("dest", formats);

            // now the input object
            JSONObject input = new JSONObject();
            input.put("input", sources);

            String uploadRequestJSON = input.toJSONString();

            // upload the JSON request and get the result
            // Send post request
            workFileConnection.setDoOutput(true);
            DataOutputStream wr = new DataOutputStream(workFileConnection.getOutputStream());

            wr.writeBytes(uploadRequestJSON);

            wr.flush();
            wr.close();

            /*
			 * Use this for error correction if there's a problem connecting
             */
            /*
			 * int responseCode = workFileConnection.getResponseCode();
			 * System.out.println("\nSending 'POST' request to URL : " + url);
			 * System.out.println("Post parameters : " + urlParameters);
			 * System.out.println("Response Code : " + responseCode);
             */
            // get the response back - this will be a JSON file
            BufferedReader in = new BufferedReader(new InputStreamReader(workFileConnection.getInputStream()));
            String inputLine;
            StringBuffer response = new StringBuffer();

            while ((inputLine = in.readLine()) != null) {
                response.append(inputLine);
            }
            in.close();

            // print result
            // System.out.println(response.toString());
            return response.toString();

        } catch (MalformedURLException e) {
            // do something with the exception
            e.printStackTrace();
            System.out.println(e);
        } catch (ProtocolException e) {
            // do something with the exception
            e.printStackTrace();
            System.out.println(e);
        } catch (IOException e) {
            // do something with the exception
            e.printStackTrace();
            System.out.println(e);
        }

        // there was a problem so return nothing
        return "";

    }

    /*
     * ContentConvertStatus
     * 
     * Uses the Accusoft PrizmDoc Content Conversion API to request
     * the status of an existing Conversion.
     *
     * Please see the Accusoft PrizmDoc Content Conversion API
     * documentation for more details:
     *
     * http://help.accusoft.com/SAAS/pcc-for-acs/webframe.html#content-conversion-service.html
     *
     * Parameters:
     * String processId :     The processId of the conversion underway.  Returned
     *                  as part of the JSON object returned at the Content
     *                  Conversion request.
     * String affinityToken:  The Accusoft-Affinity-Token returned as part as part of the JSON object returned at the Content
     *                  Conversion request.
     *
     * Returns:  A JSON object of the current Content Conversion status.
     *
     */
    public String ContentConvertStatus(String affinityToken, String processId) {
        // make sure the API key isn't blank
        if (IsApiBlank()) {

            return "API key is blank, please start again.";
        }

        // the URI to upload - includes the processId
        String url = "https://api.accusoft.com/v2/contentConverters/" + processId;

        // set the headers
        try {
            URL workFileURL = new URL(url);

            HttpsURLConnection workFileConnection = (HttpsURLConnection) workFileURL.openConnection();
            workFileConnection.setRequestMethod("GET");
            workFileConnection.setRequestProperty("acs-api-key", this.ApiKey);
            workFileConnection.setRequestProperty("Accusoft-Affinity-Token", affinityToken);

            // perform the get
            int responseCode = workFileConnection.getResponseCode();

            // get the response back - this will be a JSON file
            BufferedReader in = new BufferedReader(new InputStreamReader(workFileConnection.getInputStream()));

            String inputLine;
            StringBuffer response = new StringBuffer();

            while ((inputLine = in.readLine()) != null) {
                response.append(inputLine);
            }
            in.close();

            // print result
            // System.out.println(response.toString());
            return response.toString();

        } catch (MalformedURLException e) {
            // do something with the exception
            e.printStackTrace();
            System.out.println(e);
        } catch (ProtocolException e) {
            // do something with the exception
            e.printStackTrace();
            System.out.println(e);
        } catch (IOException e) {
            // do something with the exception
            e.printStackTrace();
            System.out.println(e);
        }

        return "";

    }

    /*
     * WorkFileDownload
     * 
     * Uses the Accusoft PrizmDoc Content Conversion API to download a
     * Work File through the Work File API.
     *
     * Please see the Accusoft PrizmDoc Work Files API
     * documentation for more details:
     *
     * http://help.accusoft.com/SAAS/pcc-for-acs/work-files.html#get-data
     *
     * Parameters:
     *  String affinityToken:  The Accusoft-Affinity-Token returned as part as part of the JSON object returned at the Content
     *                  Conversion request.
     *
     *  String fileId:          The fileId of the Work File.  In this case, this was
     *                    returned as part of the ContentConvertStatus method
     *                    to show the converted fileId.
     *
     *  
     *
     * Returns:  "Success" if everything works right, "Failure" if not.
     *
     */
    public String WorkFileDownload(String affinityToken, String fileId, String output) {
        // make sure the API key isn't blank
        if (IsApiBlank()) {

            return "API key is blank, please start again.";
        }

        // the URI to upload - includes the processId
        String url = "https://api.accusoft.com/PCCIS/V1/WorkFile/" + fileId;

        // set the headers
        try {
            URL workFileURL = new URL(url);

            HttpsURLConnection workFileConnection = (HttpsURLConnection) workFileURL.openConnection();
            workFileConnection.setRequestMethod("GET");
            workFileConnection.setRequestProperty("acs-api-key", this.ApiKey);
            workFileConnection.setRequestProperty("Accusoft-Affinity-Token", affinityToken);

            ReadableByteChannel rbc = Channels.newChannel(workFileConnection.getInputStream());
            FileOutputStream convertedFile = new FileOutputStream(output);
            convertedFile.getChannel().transferFrom(rbc, 0, Long.MAX_VALUE);

            convertedFile.flush();
            convertedFile.close();
            rbc.close();

            return "Success.";

        } catch (MalformedURLException e) {
            // do something with the exception
            e.printStackTrace();
            System.out.println(e);
        } catch (ProtocolException e) {
            // do something with the exception
            e.printStackTrace();
            System.out.println(e);
        } catch (IOException e) {
            // do something with the exception
            e.printStackTrace();
            System.out.println(e);
        }

        return "Failure.";

    }

    /*
	 * getUploadFileID
	 * 
	 * Takes in the JSON file as part of the Accusoft Work File API
	 * Upload method and extracts the fileId.
	 * 
	 * Parameters:  
	 * JSONResponse:	String as a JSON file
	 * 
	 * Returns fileId as a String
	 * 
     */
    public static String getUploadFileID(String JSONResponse) {
        JSONParser parser = new JSONParser();
        try {
            Object obj = parser.parse(JSONResponse);
            JSONObject readConfig = (JSONObject) obj;

            return (String) readConfig.get("fileId");

        } catch (ParseException e) {
            e.printStackTrace();
            // end it here kids
            System.exit(0);
        }

        // there was a problem
        return "ERROR";

    }

    /*
	 * 
	 * getUploadAffinityToken
	 * 
	 * Takes in the JSON file as part of the Accusoft Work File API
	 * Upload method and extracts the fileId.
	 * 
	 * Parameters:  
	 * JSONResponse:	String as a JSON file
	 * 
	 * Returns affinityToken as a String
	 * 
     */
    public static String getUploadAffinityToken(String JSONResponse) {
        JSONParser parser = new JSONParser();
        try {
            Object obj = parser.parse(JSONResponse);
            JSONObject readConfig = (JSONObject) obj;

            return (String) readConfig.get("affinityToken");

        } catch (ParseException e) {
            e.printStackTrace();
            // end it here kids
            System.exit(0);
        }

        // there was a problem
        return "ERROR";

    }

    /*
	 * 
	 * getConvertProcessId
	 * 
	 * Takes in the JSON file as part of the Accusoft Content Conversion API
	 * Upload method and extracts the fileId.
	 * 
	 * Parameters:  
	 * JSONResponse:	String as a JSON file
	 * 
	 * Returns processId as a String
	 * 
     */
    public static String getConvertProcessId(String JSONResponse) {
        JSONParser parser = new JSONParser();
        try {
            Object obj = parser.parse(JSONResponse);
            JSONObject readConfig = (JSONObject) obj;

            return (String) readConfig.get("processId");

        } catch (ParseException e) {
            e.printStackTrace();
            // end it here kids
            System.exit(0);
        }

        // there was a problem
        return "ERROR";

    }

    /*
	 * 
	 * getConvertState
	 * 
	 * Takes in the JSON file as part of the Accusoft Content Conversion API
	 * Upload method and extracts the fileId.
	 * 
	 * Parameters:  
	 * JSONResponse:	String as a JSON file
	 * 
	 * Returns state as a String
	 * 
     */
    public static String getConvertState(String JSONResponse) {

        try {
            JSONParser parser = new JSONParser();
            Object obj = parser.parse(JSONResponse);
            JSONObject readConfig = (JSONObject) obj;

            return (String) readConfig.get("state");

        } catch (ParseException e) {
            e.printStackTrace();
            // end it here kids
            System.exit(0);
        }

        // there was a problem
        return "ERROR";

    }

    /*
	 * 
	 * getDownloadFileId
	 * 
	 * Takes in the JSON file as part of the Accusoft Content Conversion API
	 * Upload method and extracts the fileId.
	 * 
	 * Parameters:  
	 * JSONResponse:	String as a JSON file
	 * 
	 * Returns output as a String
	 * 
     */
    public static String getDownloadFileId(String JSONResponse) {
        try {
            JSONParser parser = new JSONParser();
            Object obj = parser.parse(JSONResponse);
            JSONObject readConfig = (JSONObject) obj;

            // drill down
            JSONObject convertOutput = (JSONObject) readConfig.get("output");

            JSONArray convertResults = (JSONArray) convertOutput.get("results");

            JSONObject convertResults0 = (JSONObject) convertResults.get(0);

            return (String) convertResults0.get("fileId");

        } catch (ParseException e) {
            e.printStackTrace();
        }

        // there was a problem
        return "ERROR";
    }

    /*
	 * Arguments:
	 * InputFile:  	The file name to be read in and converted to PDF.
	 * OutputFile:  The file name to be written as PDF.
     */
    public static void main(String[] args) {
        // TODO Auto-generated method stub
        System.out.println("Starting the PrizmDoc demo in Java.");

        if (args.length != 2) {
            System.out.println("Requires 2 arguments:  inputFile and outputFile.");
        }

        // Initiate the PrizmDocConverter object
        String uploadFile = args[0];
        String downloadFile = args[1];

        PrizmDocConverter newDocUpload = new PrizmDocConverter(getAPIKey());

        String response = "";
        response = newDocUpload.WorkFileUpload(uploadFile);
        System.out.println(response);

        // now get the pieces we want
        String uploadFileId = getUploadFileID(response);

        String uploadAffinityToken = getUploadAffinityToken(response);

        System.out.println("uploadFileId: " + uploadFileId);
        System.out.println("uploadAffinityToken: " + uploadAffinityToken);

        // time to download our file
        response = "";
        response = newDocUpload.WorkFileDownload(uploadAffinityToken, uploadFileId, "test.docx");
        System.out.println(response);

        // issue the conversion request
        response = "";
        response = newDocUpload.ContentConvert(uploadAffinityToken, uploadFileId, "pdf");
        System.out.println(response);

        // get the fields we need from this response
        // extract the processId and the state
        String convertProcessId = getConvertProcessId(response);
        String convertState = getConvertState(response);

        System.out.println("processId: " + convertProcessId);
        System.out.println("state: " + convertState);

        // while look - check the status every 10 seconds.
        // when state = "complete" then stop
        while (!convertState.equals("complete")) {
            /*
                In the event of an error during the conversion process,
                print out the JSON response and exit the program.
            */
            
            if(convertState.equals("error"))
            {
                System.out.println("There has been an error in conversion: ");
                System.out.println(response);
                System.exit(0);
            }
            try {
                TimeUnit.SECONDS.sleep(10);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }

            response = "";
            response = newDocUpload.ContentConvertStatus(uploadAffinityToken, convertProcessId);

            convertState = getConvertState(response);
            System.out.println("ConvertState: \n");
            System.out.println(convertState);

        }

        // ok - now get the new fileId to download
        String downloadFileId = getDownloadFileId(response);

        System.out.println("downloadFileId: " + downloadFileId);

        // time to download our file
        response = "";
        response = newDocUpload.WorkFileDownload(uploadAffinityToken, downloadFileId, downloadFile);
        System.out.println(response);
    }

}
